#include "pass_rule.h"
/**********************************
* pass_rule.c
*
* if pass_rule return TRUE, the 
* packet is not filtered.  If 
* it returns FALSE the packet is
* filtered
**********************************/
#include "../snort-1.7/snort.h"
#include "../snort-1.7/decode.h"
#include "../snort-1.7/rules.h"
#include "../snort-1.7/parser.h"
#include <pthread.h>

extern ListHead* head_tmp;

pthread_mutex_t snort_mutex = PTHREAD_MUTEX_INITIALIZER;

void pass_rule_cmd_line(int argc, char* argv[]){
	int i=0;
	
	for (i=0; i<argc; i++){
		if (strncasecmp("-c", argv[i], 2)==0){
			if ( (i>=argc-1) || (argv[i+1][0]=="-"[0]) ){
				printf("Syntax -c <rules file>\n");
				exit(1);
			}
			strncpy(pv.config_file, argv[i+1], STD_BUF-1);
			pv.use_rules=1;
			pv.rules_order_flag=1;
			OrderRuleLists("pass alert log");
			ParseRulesFile(pv.config_file, 0);
		}else if (strncasecmp("-l", argv[i], 2)==0){
			if ( (i>=argc-1) || (argv[i+1][0]=="-"[0]) ){
				printf("Syntax -l <log directory>\n");
				exit(1);
			}
			strncpy(pv.log_dir, argv[i+1], STD_BUF-1);
			pv.log_flag=1;
		}
	}
	
	if (!pv.use_rules){
		printf("You must specify a rules file with -c <rules file>\n");
		exit(1);
	}
}

/*********************************************************
* Add the rules we can't work without to snort
*********************************************************/
void add_mandatory_rules(){
	char		Rule[1024];
	ListHead*	listhead=NULL;
	
	printf("***********Adding mandatory rules****************\n");
	sprintf(Rule,"output scrub_alert\n");
   	/*ParseRule(Rule,0);*/
	
	listhead = CreateRuleType("drop", RULE_ALERT, 1, NULL);
	head_tmp = listhead;
	ParseOutputPlugin(Rule);
	head_tmp=NULL;
}

void pass_rule_init(pcap_t* pad, int argc, char* argv[]){
	grinders[0] = DecodeEthPkt;	
	pd=pad;	

	/*from snort main*/
	InitNetmasks();
	InitProtoNames();
	
	strncpy(pv.log_dir, "/var/log/snort", STD_BUF-1);
	pv.log_flag=1;
 
    InitPreprocessors();
    InitPlugIns();
    InitOutputPlugins();
	
	/* setup the default rule action anchor points */
    CreateDefaultRules();
	
	add_mandatory_rules();
	pass_rule_cmd_line(argc, argv);
	
	
	/*strncpy(pv.config_file, "rules.test", 11);
	pv.use_rules = 1;
	pv.rules_order_flag=1;
	OrderRuleLists("pass alert log");
	ParseRulesFile(pv.config_file, 0);*/
	
	alert = OpenAlertFile("/var/log/snort/alert");
	
	pv.log_cmd_override=1;
	pv.data_flag=1;
	pv.show2hdr_flag=1;
	pv.logbin_flag=0;
}

int pass_rule(char *user, struct pcap_pkthdr * pkthdr, u_char * pkt){
	Packet 	p;
	int 	return_value;

	pthread_mutex_lock(&snort_mutex);
	FilterThis=FALSE;
	(*grinder)(&p, pkthdr, pkt);
	p.interface_num=(int)user;
	/*if (p.tcph)
	if (p.tcph->th_flags & TH_SYN){
		printf("TCP SYN Flags\n");
	}*/
	/*if (p.iph){
		printf("%s->", inet_ntoa(p.iph->ip_src));
		printf("%s\n", inet_ntoa(p.iph->ip_dst));
	}*/

	Preprocess(&p);
	ClearDumpBuf();
	return_value = !FilterThis;
	pthread_mutex_unlock(&snort_mutex);
		
	return return_value;
}

void pass_rule_free(){
}
